////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                //
// HEL Library (c) 2003-2005 by Peter Schraut                                                     //
// http://www.console-dev.de                                                                      //
//                                                                                                //
// ---------------------------------------------------------------------------------------------- //
//                                                                                                //
// HEL Library is distributed as freeware.                                                        //
//                                                                                                //  
// You are free to use HEL Library as part of your program for any purpose including freeware,    //
// commercial and shareware programs, provided some credit is given to the author.                //
//                                                                                                // 
// The origin of this software must not be misrepresented; you must not claim your authorship.    //
// All redistributions must retain the original copyright notice and web site addresses.          //
//                                                                                                // 
// Commercial redistribution of the library is allowed only with an explicit written              // 
// permission from the author.                                                                    // 
//                                                                                                // 
// If you finished a commercial game using HEL library, the author would like to be informed      // 
// in what game it is used.                                                                       // 
//                                                                                                // 
// This software is provided 'as-is', without warranty of any kind, either expressed or implied.  // 
// In no event shall the author be held liable for any damages arising from the use of this       //
// software.                                                                                      // 
//                                                                                                // 
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef HEL_H
#define HEL_H

#include <mygba.h>
#include <stdio.h>
#include <stdarg.h>
#include <malloc.h>

// Define HAM_DEBUGGER when generating the
// documentation using DoxyGen. This is only
// that DoxyGen uses the function documentation,
// since we do not have documented the macros
#ifdef DOXYGEN_SHOULD_SKIP_THIS
  #define HAM_DEBUGGER 1
#endif

//
// ---[ HEL version defines ]-----------------------------------------------------------------------
//

#define HEL_CURRENT_VERSION_MAJOR   1
#define HEL_CURRENT_VERSION_MINOR   3
#define HEL_CURRENT_VERSION_STRING  "1.3 Final"


//
// ---[ Version checking macros for HAM ]-----------------------------------------------------------
//

//
// EQ=Equal, NE=NotEqual, GT=GreaterThan, GTE=GreaterThanEqual, LE=LessEqual, LT=LessThan
//
#define CURRENT_HAM_VERSION_EQ(major, minor)   (HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR >= (minor))
#define CURRENT_HAM_VERSION_NE(major, minor)   (HAM_CURRENT_VERSION_MAJOR != (major) || HAM_CURRENT_VERSION_MINOR != (minor))
#define CURRENT_HAM_VERSION_GT(major, minor)   ((HAM_CURRENT_VERSION_MAJOR > (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR > (minor))?TRUE:FALSE)
#define CURRENT_HAM_VERSION_GTE(major, minor)  ((HAM_CURRENT_VERSION_MAJOR > (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR >= (minor))?TRUE:FALSE)
#define CURRENT_HAM_VERSION_LE(major, minor)   ((HAM_CURRENT_VERSION_MAJOR < (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR <= (minor))?TRUE:FALSE)
#define CURRENT_HAM_VERSION_LT(major, minor)   ((HAM_CURRENT_VERSION_MAJOR < (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR < (minor))?TRUE:FALSE)


//
// ---[ General Typedef's ]-------------------------------------------------------------------------
//

typedef struct
{
   s8 X, Y;
}TPoint8;

typedef struct
{
   u8 X, Y;
}TPointU8;

typedef struct
{
   s16 X, Y;
}TPoint16;

typedef struct
{
   u16 X, Y;
}TPointU16;

typedef struct
{
   s32 X, Y;
}TPoint32;

typedef struct
{
   u32 X, Y;
}TPointU32;

typedef struct
{
   s8 Left;
   s8 Right;
   s8 Top;
   s8 Bottom;
}TRect8;

typedef struct
{
   s16 Left;
   s16 Right;
   s16 Top;
   s16 Bottom;
}TRect16;

typedef struct
{
   s32 Left;
   s32 Right;
   s32 Top;
   s32 Bottom;
}TRect32;



//
// ---[ Defines ]-----------------------------------------------------------------------------------
//
#define NOT                            !
#define SAME(a,b)                      ((a) == (b))
#define BIT(n)                         (1<<(n))
#define BIT_NOT(n)                     (~BIT(n))
#define BIT_IS_SET(v,b)                ((((v) >> (b)) & 1))
#define SHR(v,n)                       ((v) >> (n))
#define SHL(v,n)                       ((v) << (n))
#define MIN(a,b)                       (((a)<(b))?(a):(b))
#define MID(a,b,c)                     (MAX(MIN((a),(b)),MIN((a),(c)))
#define MAX(a,b)                       (((a)>(b))?(a):(b))
#define LIMIT(Value,ValueMin,ValueMax) (MIN(MAX((Value), (ValueMin)), (ValueMax)))
#define SGN(v)                         (((v) < 0)?-1:(((v) > 0)?(+1):(0)))
#define ABS(v)                         (((v) > 0)?(v):(-(v)))
#define ASSIGNED(p)                    (NULL != (p))
#define ACCESS_U8(v)                   *(volatile u8*)(v)
#define ACCESS_U16(v)                  *(volatile u16*)(v)
#define ACCESS_U32(v)                  *(volatile u32*)(v)
#define RANDOM(n)                      (rand()%(n))
#define SIN(n)                         (SIN_SFP32[(n)%360])
#define COS(n)                         (COS_SFP32[(n)%360])
#define ATTR_ALIGNED(n)                __attribute__ ((aligned(n)))
#define ATTR_PACKED                    __attribute__ ((packed))
#define ATTR_SECTION(name)             __attribute__ ((section(name)))
#define ATTR_MEM_IN_EWRAM              ATTR_SECTION(".ewram")
#define ATTR_MEM_IN_IWRAM              ATTR_SECTION(".iwram")
#define ATTR_FUNC_IN_IWRAM             __attribute__ ((section (".iwram"), long_call))
#define FREE(p)                        { if(ASSIGNED(p)){ free(p); (p)=NULL; } }
#define EXTERN_DATA8(name,size)        extern const u8 name[size];
#define EXTERN_DATA16(name,size)       extern const u16 name[size];
#define EXTERN_DATA32(name,size)       extern const u32 name[size];
#define EXTERN_U8(name,size)           extern const unsigned char name[size];
#define EXTERN_U16(name,size)          extern const unsigned short name[size];
#define EXTERN_U32(name,size)          extern const unsigned long name[size];
#define INCBIN(file)                   __asm__ (".incbin \"" file "\"\n");
#define INCBIN_U8(array, file)         const unsigned char array[]={};  INCBIN(file);
#define INCBIN_U16(array, file)        const unsigned short array[]={}; INCBIN(file);
#define INCBIN_U32(array, file)        const unsigned long array[]={};  INCBIN(file);


//
// Use it like:
//  TMyStruct *p;
//      p = NEW(TMyStruct, 10); // allocs memory for 10 TMyStruct's
//
// use HAM's malloc wrapper if HAM version >= 2.80 and HAM_DEBUGGER is enabled
#if CURRENT_HAM_VERSION_GTE(2,80) && HAM_DEBUGGER
   #define NEW(struct_, count)         (struct_*)ham_Malloc((count)*sizeof(struct_))
#else
   #define NEW(struct_, count)         (struct_*)malloc((count)*sizeof(struct_))
#endif


// VBAText stuff does -NOT- work on hardware!!!
// Hint: HEL_DEBUGMSG instead!
#ifdef HAM_DEBUGGER
   #define VBAOUT(txt, args...)        ham_VBAText(txt,##args);
#else
   #define VBAOUT(txt, args...)
#endif


#define SET_POINT(P, X_, Y_)        \
   {                                \
      (P)->X = X_; (P)->Y = Y_;     \
   }

#define SET_RECT(P, Left_, Top_, Right_, Bottom_)  \
   {                                               \
      (P)->Left=Left_;   (P)->Top=Top_;            \
      (P)->Right=Right_;  (P)->Bottom=Bottom_;     \
   }




#ifndef __cplusplus
   #define false     0
   #define true      (!false)
   
   #ifndef bool
      #define bool    u8
   #endif 
   
   #ifndef null
      #define null    NULL
   #endif 
#endif


// These can be used with TOOL_DMA0/1/2/3_SET_EX
#define DMA_INCREMENT       (0)
#define DMA_DECREMENT       (1)
#define DMA_NOCHANGE        (2)
#define DMA_REPEAT_ON       (1)
#define DMA_REPEAT_OFF      (0)
#define DMA_IRQ_ON          (1)
#define DMA_IRQ_OFF         (0)


#if CURRENT_HAM_VERSION_LT(2,80)
   // can be used with ham_InitMapEmptySet(..)
   // normal map sizes
   #define MAP_SIZE_32X32        0
   #define MAP_SIZE_64X32        1
   #define MAP_SIZE_32X64        2
   #define MAP_SIZE_64X64        3
   
   // rotationmap sizes
   #define MAP_SIZE_ROT_16X16    0
   #define MAP_SIZE_ROT_32X32    1
   #define MAP_SIZE_ROT_64X64    2
   #define MAP_SIZE_ROT_128X128  3
#endif


//#if CURRENT_HAM_VERSION_LT(2,80)
   #define OBJ_COLORS_16         0
   #define OBJ_COLORS_256        1
   //#define OBJ_INVALID_ID        0xff
//#endif


// Helper defines. Can be used for hel_Splash(..)
#define SPLASH_TYPE_BLACK        0
#define SPLASH_TYPE_WHITE        1


// Helper defines for hel_FadePal(..)
#define FADE_PALETTE_BG          0
#define FADE_PALETTE_OBJ         1
#define FADE_COLORS_16           0
#define FADE_COLORS_256          1


// Helper defines for windowing
#define WIN_ALL                  (WIN_BG0 | WIN_BG1 | WIN_BG2 | WIN_BG3 | WIN_OBJ)
#define WIN_NONE                 (0)
// specify 0 or 1 to whether exclude or include window
#define WIN_SELECT(bg0, bg1, bg2, bg3, obj)   \
   ( (WIN_BG0*(bg0)) | (WIN_BG1*(bg1)) | (WIN_BG2*(bg2)) | (WIN_BG3*(bg3)) | (WIN_OBJ*(obj)) )


//
// ---[ SpecialFX Defines ]---------------------------------------------------------------------
//
// These defines can be used with ham_SetFxMode
//
#define FX_LAYER_BG0     BIT0
#define FX_LAYER_BG1     BIT1
#define FX_LAYER_BG2     BIT2
#define FX_LAYER_BG3     BIT3
#define FX_LAYER_OBJ     BIT4
#define FX_LAYER_BD      BIT5
#define FX_LAYER_ALL     0x3f
#define FX_LAYER_NONE    0x00


//
// ---[ Generic Screen Defines ]---------------------------------------------------------------------
//
#define SCREEN_DISPLAY_WIDTH              240
#define SCREEN_DISPLAY_HEIGHT             160

//
// ---[ Parallax macros ]---------------------------------------------------------------------------
//
#define PARALLAX_RATIO_ONE                0xffff
#define PARALLAX_FLOAT_TO_RATIO(value)    CONV_FLOAT_TO_SFP32(value)
#define PARALLAX_RATIO_TO_FLOAT(value)    (float)((value)/(float)PARALLAX_RATIO_ONE)
#define PARALLAX_RATIO_TO_INT(value)      CONV_SFP32_TO_S16(value)


//
// ---[ Background macros ]-------------------------------------------------------------------------
//
#define M_BGXSCRLX_SET(bgno, xoff)        ACCESS_16(MEM_IO + 0x10 + (bgno<<2)) = xoff;
#define M_BGXSCRLY_SET(bgno, yoff)        ACCESS_16(MEM_IO + 0x12 + (bgno<<2)) = yoff;
#define M_BGXSCRLXY_SET(bgno, xoff, yoff) ACCESS_16(MEM_IO + 0x10 + (bgno<<2)) = xoff; \
                                          ACCESS_16(MEM_IO + 0x12 + (bgno<<2)) = yoff;


//
// ---[ Timer macros ]------------------------------------------------------------------------------
//
#define F_TIM0CNT_IRQ_ENABLED                ((R_TIM0CNT & BIT6) == BIT6)
#define F_TIM0CNT_TIMER_STARTED              ((R_TIM0CNT & BIT7) == BIT7)
#define M_TIM1CNT_COUNTUP_TIMING_ENABLE      R_TIM1CNT &= NOT_BIT2; R_TIM1CNT |= BIT2;
#define M_TIM1CNT_COUNTUP_TIMING_DISABLE     R_TIM1CNT &= NOT_BIT2;
#define F_TIM1CNT_COUNTUP_TIMING_ENABLED     ((R_TIM1CNT &= NOT_BIT2) == NOT_BIT2)
#define F_TIM1CNT_IRQ_ENABLED                ((R_TIM1CNT & BIT6) == BIT6)
#define F_TIM1CNT_TIMER_STARTED              ((R_TIM1CNT & BIT7) == BIT7)
#define M_TIM2CNT_COUNTUP_TIMING_ENABLE      R_TIM2CNT &= NOT_BIT2; R_TIM2CNT |= BIT2;
#define M_TIM2CNT_COUNTUP_TIMING_DISABLE     R_TIM2CNT &= NOT_BIT2;
#define F_TIM2CNT_COUNTUP_TIMING_ENABLED     ((R_TIM2CNT &= NOT_BIT2) == NOT_BIT2)
#define F_TIM2CNT_IRQ_ENABLED                ((R_TIM2CNT & BIT6) == BIT6)
#define F_TIM2CNT_TIMER_STARTED              ((R_TIM2CNT & BIT7) == BIT7)
#define M_TIM3CNT_COUNTUP_TIMING_ENABLE      R_TIM3CNT &= NOT_BIT2; R_TIM3CNT |= BIT2;
#define M_TIM3CNT_COUNTUP_TIMING_DISABLE     R_TIM3CNT &= NOT_BIT2;
#define F_TIM3CNT_COUNTUP_TIMING_ENABLED     ((R_TIM3CNT &= NOT_BIT2) == NOT_BIT2)
#define F_TIM3CNT_IRQ_ENABLED                ((R_TIM3CNT & BIT6) == BIT6)
#define F_TIM3CNT_TIMER_STARTED              ((R_TIM3CNT & BIT7) == BIT7)


//
// ---[ Helper macros ]-----------------------------------------------------------------------------
//

//
// Background creation macro for gfx2gba nameconventions.
// For example:
//             H_CREATE_BG(Background, 240, 160, 1, 0, 2, 0);
//
// Example creates a 240x160 pixel background on layer1, using 16 colors
// and prio2, no rotation map.
// Arrays are called Background_Tiles and Background_Map
//
#define H_BG_CREATE(ArrayName, Width, Height, BgNo, ColMode, Prio, RotationMap) \
    {                                                                           \
        u8 MapSize=0;                                                           \
        map_fragment_info *pMapInfo=NULL;                                       \
                                                                                \
        if(!RotationMap)                                                        \
        {                                                                       \
            if(Width <= 256 && Height <= 256)       MapSize=0;                  \
            else if(Width <= 512  && Height <= 256) MapSize=1;                  \
            else if(Width <= 256 && Height <= 512)  MapSize=2;                  \
            else if(Width <= 512 && Height <= 512)  MapSize=3;                  \
        }                                                                       \
        else                                                                    \
        {                                                                       \
            if(Width <= 128 && Height <= 128)        MapSize=0;                 \
            else if(Width <= 256  && Height <= 256)  MapSize=1;                 \
            else if(Width <= 512  && Height <= 512)  MapSize=2;                 \
            else if(Width <= 1024 && Height <= 1024) MapSize=3;                 \
        }                                                                       \
                                                                                \
        ham_bg[BgNo].ti = ham_InitTileSet((void*)ArrayName##_Tiles,             \
                            SIZEOF_16BIT(ArrayName##_Tiles), ColMode, 1);       \
                                                                                \
        ham_bg[BgNo].mi = ham_InitMapEmptySet(MapSize, RotationMap);            \
                                                                                \
        pMapInfo = ham_InitMapFragment((void*)ArrayName##_Map, Width/8,         \
                        Height/8,  0, 0, Width/8, Height/8, RotationMap);       \
                                                                                \
        ham_InsertMapFragment(pMapInfo, BgNo, 0, 0);                            \
        ham_InitBg(BgNo, FALSE, Prio, FALSE);                                   \
        pMapInfo = NULL;                                                        \
    }


#define H_BG_DEINIT_SELECT(bg0, bg1, bg2, bg3)                                  \
    {                                                                           \
        if(bg0) { ham_SetBgXY(0, 0, 0); ham_DeInitBg(0); }                      \
        if(bg1) { ham_SetBgXY(1, 0, 0); ham_DeInitBg(1); }                      \
        if(bg2) { ham_SetBgXY(2, 0, 0); ham_DeInitBg(2); }                      \
        if(bg3) { ham_SetBgXY(3, 0, 0); ham_DeInitBg(3); }                      \
    }


#define H_BG_VISIBLE_SELECT(bg0, bg1, bg2, bg3)                                 \
    {                                                                           \
        if(bg0) ham_SetBgVisible(0, TRUE);                                      \
        if(bg1) ham_SetBgVisible(1, TRUE);                                      \
        if(bg2) ham_SetBgVisible(2, TRUE);                                      \
        if(bg3) ham_SetBgVisible(3, TRUE);                                      \
    }


#define H_BG_INVISIBLE_SELECT(bg0, bg1, bg2, bg3)                               \
    {                                                                           \
        if(bg0) ham_SetBgVisible(0, FALSE);                                     \
        if(bg1) ham_SetBgVisible(1, FALSE);                                     \
        if(bg2) ham_SetBgVisible(2, FALSE);                                     \
        if(bg3) ham_SetBgVisible(3, FALSE);                                     \
    }


#define H_FULL_DARKEN                                                           \
    {                                                                           \
        ham_SetFxMode(FX_LAYER_SELECT(1,1,1,1,1,1),                             \
                      FX_LAYER_SELECT(1,1,1,1,1,1),                             \
                      FX_MODE_DARKEN);                                          \
                                                                                \
        ham_SetFxBrightnessLevel(16);                                           \
    }


#define H_WAIT_VSYNC { while((volatile u16)REG_VCOUNT != 160){} }




#ifdef __cplusplus
   extern "C" {
#endif

//
// ---[ Forward declarations ]----------------------------------------------------------------------
//
struct _TMapScrollInfo; 

//
// ---[ Map functions ]-----------------------------------------------------------------------------
//
#define BG_VIRTUAL   0x10

typedef void (*PMapNotifyFunc)(struct _TMapScrollInfo *pMapInfo, u32 X, u32 Y);

typedef struct
{
   PMapNotifyFunc pOnTopRowChanged;
   PMapNotifyFunc pOnLeftColumnChanged;
}TMapNotifier, *PMapNotifier;

typedef struct _TMapParallaxInfo
{
   TPoint32 Ratio;                  // The Parallax-Ratio we use to Multiply the ScrollValue with
   TPoint32 CurrentAmount;          // Current amount, distance, since the last move
}TMapParallaxInfo, *PMapParallaxInfo;

typedef struct _TMapScrollInfo
{
   u8 BgNo;                         // The Backgroundnumber this structure is assigned  to
   TPoint16 Size;                   // The size of the map in tiles
   void *pData;                     // Pointer to mapdata
   TPoint16 Position;               // The current scrollposition in tiles
   TPoint8 Runner;                  // The current scrollposition between current and next tile, in pixel
   u8 BoundsCheck;                  // Flag to enable/disable boundschecking
   TMapNotifier MapRedrawNotifiers; // Callback functions
   u8 UseParallax;                  // Flag to enable/disable Parallax-Scrolling (added in v1.2)
   TMapParallaxInfo ParallaxInfo;   // Additional information for Parallax-Scrolling (added in v1.2)
   u8 DataTypeSize;                 // The datatypesize of the data assigned to a VisualMap (added in v1.3 beta 1)
   void *pCustomData;               // A pointer to whatever data the user want to point to (added in v1.3 beta 2)
}TMapScrollInfo, *PMapScrollInfo;

extern TMapScrollInfo hel_MapInit(u8 BgNo, u16 TilesX, u16 TilesY, u8 Rotation, void *pMapData);
extern TMapScrollInfo hel_MapInitVirtual(u16 TilesX, u16 TilesY, u8 DataTypeSize, void *pMapData);
extern void hel_MapDeInit(TMapScrollInfo *pMapInfo);
extern TMapScrollInfo hel_MapInitEx(u8 BgNo, u16 TilesX, u16 TilesY, u8 Rotation, u8 MapSize, void *pMapData);
extern void hel_MapSetCallbacks(TMapScrollInfo *pMapInfo, PMapNotifyFunc pOnTopRowChanged, PMapNotifyFunc pOnLeftColumnChanged);
extern u8 hel_MapScrollBy(TMapScrollInfo *pMapInfo, s32 DeltaX, s32 DeltaY);
extern u8 hel_MapIsBoundsCheckEnabled(TMapScrollInfo *pMapInfo);
extern void hel_MapSetBoundsCheck(TMapScrollInfo *pMapInfo, u8 Value);
extern void hel_MapSetPosition(TMapScrollInfo *pMapInfo, u32 X, u32 Y);
extern void hel_MapSetPositionInPixel(TMapScrollInfo *pMapInfo, u32 X, u32 Y);
extern TPoint32 hel_MapGetPosition(TMapScrollInfo *pMapInfo);
extern TPoint32 hel_MapGetPositionInPixel(TMapScrollInfo *pMapInfo);
extern void hel_MapRedraw(TMapScrollInfo *pMapInfo);
extern void hel_MapBatchScrollBy(TMapScrollInfo *pMapInfo, s32 DeltaX, s32 DeltaY, int Count);
extern void hel_MapSetParallaxRatio(TMapScrollInfo *pMapInfo, FIXED RatioX, FIXED RatioY);
extern void hel_MapSetParallax(TMapScrollInfo *pMapInfo, u8 Value);
extern u16 hel_MapGetTileAt(TMapScrollInfo *pMapInfo, u32 ScreenX, u32 ScreenY);
extern void *hel_MapGetTilePtrAt(TMapScrollInfo *pMapInfo, u32 ScreenX, u32 ScreenY);
extern TPoint32 hel_MapGetPositionInPixelFrom(TMapScrollInfo *pMapInfo, u32 TileX, u32 TileY);
extern u8 hel_MapBatchScrollByEx(TMapScrollInfo *pMapInfos, s32 DeltaX, s32 DeltaY, int Count, u32 PrimaryLayer);
extern void hel_MapSetCustomData(TMapScrollInfo *pMapInfo, void *pCustomData);
extern void *hel_MapGetCustomData(TMapScrollInfo *pMapInfo);

//
// ---[ Map Camera functions ]----------------------------------------------------------------------
//

#define MAP_CAM_NEXTKEYPOINT    0x11
#define MAP_CAM_LASTKEYREACHED  0x12

typedef TPoint16 TMapCamKeyPoint;
typedef TMapCamKeyPoint *PMapCamKeyPoint;


typedef struct _TMapCamScrollInfo
{
   TMapCamKeyPoint *pKeyPoints;
   u16 nKeyPoints;
   u16 CurKeyPoint;
}TMapCamScrollInfo, *PMapCamScrollInfo;

extern u8 hel_MapScrollTo(TMapScrollInfo *pMapInfo, TMapCamScrollInfo *pMapCamInfo, u32 SpeedX, u32 SpeedY);



//
// ---[ Text functions ]----------------------------------------------------------------------------
//

typedef struct
{
   u8 BgNo;
   char *CharOrder;
   u8 CharIndex[256];
   // fix in 0.9
   u8 ColMode;
   u8 PalNo;
}TCustomTextInfo, *PCustomTextInfo;

//extern void hel_CustomTextInit(TCustomTextInfo *pTextInfo, void *pSrcTiles, u16 SizeU16, u8 ColMode, u8 MapSize, u8 MapRot);
extern void hel_CustomTextInit(TCustomTextInfo *pTextInfo, void *pSrcTiles, u16 SizeU16, u8 ColMode, u8 PalNo, u8 MapSize, u8 MapRot);
extern void hel_CustomTextClear(TCustomTextInfo *pTextInfo);
extern void hel_CustomTextClearRow(TCustomTextInfo *pTextInfo, u32 y);
extern void hel_CustomTextPrint(TCustomTextInfo *pTextInfo, u32 x, u32 y, char *pText);
extern void hel_CustomTextPrintFmt(TCustomTextInfo *pTextInfo, u32 x, u32 y, char *pFmt, ...);
extern u8 hel_CustomTextType(TCustomTextInfo *pTextInfo, u32 x, u32 y, u8 Delay, u16 Wait, char *pText);

//
// ---[ Pad functions ]-----------------------------------------------------------------------------
//
#define PAD_BEHAVIOUR_PRESSED_ON_BUTTON_UP      0
#define PAD_BEHAVIOUR_PRESSED_ON_BUTTON_DOWN    1

typedef struct _TPadButtons
{
   u8 Left:1;
   u8 Right:1;
   u8 Down:1;
   u8 Up:1;
   u8 Start:1;
   u8 Select:1;
   u8 A:1;
   u8 B:1;
   u8 L:1;
   u8 R:1;
   u8 None:1;
}TPadButtons, *PPadButtons;

typedef struct
{
   TPadButtons Pressed;
   TPadButtons Held;
   u8 Behaviour;
}TPad, *PPad;

extern TPad hel_Pad; // pad variable

extern void hel_PadInit(void);
extern void hel_PadReset(void);
extern void hel_PadReadState(void);
extern void hel_PadSetBehaviour(u32 NewBehaviour);

//
// ---[ Palette functions ]-------------------------------------------------------------------------
//

extern void hel_PalBgInvert16(u8 PaletteBank);
extern void hel_PalBgInvert256(void);
extern void hel_PalBgInvertEx(u8 StartIndex, u8 EndIndex);
extern void hel_PalBgClear16(u8 PaletteBank);
extern void hel_PalBgClear256(void);
extern void hel_PalBgClearEx(u8 StartIndex, u8 EndIndex, u8 Red, u8 Green, u8 Blue);
extern void hel_PalObjLoad16(void *pPalette, u32 PaletteBank);

//
// ---[ Obj functions ]-----------------------------------------------------------------------------
//

extern const u32 OBJ_SIZE_LUT[3][4] ATTR_ALIGNED(4);
extern TPointU8 __hel_ObjMosaicSize;

#define __hel_ObjExists(ObjNo)              (ham_obj[ObjNo].taken == 1)
#define __hel_ObjIsHFlip(ObjNo)             ((ham_obj[ObjNo].objatr1 >> 12) & 1)
#define __hel_ObjIsVFlip(ObjNo)             ((ham_obj[ObjNo].objatr1 >> 13) & 1)
#define __hel_ObjIsVisible(ObjNo)           (ham_obj[ObjNo].active == TRUE)
#define __hel_ObjIsMosaic(ObjNo)            ((ham_obj[ObjNo].objatr0 >> 12) & 1)
#define __hel_ObjIsRotScale(ObjNo)          ((ham_obj[ObjNo].objatr0 >>  8) & 1)
#define __hel_ObjGetColorMode(ObjNo)        ((ham_obj[ObjNo].objatr0 >> 13) & 1)
#define __hel_ObjGetMode(ObjNo)             ((ham_obj[ObjNo].objatr0 >> 10) & 2)
#define __hel_ObjBringToFront(ObjNo)        ham_SetObjBefore(ObjNo, ham_obj_first)
#define __hel_ObjGetFirst()                 ham_obj_first
#define __hel_ObjHideAll()                  hel_ObjSetVisibleAll(FALSE)
#define __hel_ObjShowAll()                  hel_ObjSetVisibleAll(TRUE)
#define __hel_ObjToggleVisible(ObjNo)       ham_obj[ObjNo].active = !ham_obj[ObjNo].active
#define __hel_ObjSetVisible(ObjNo, Value)   { ham_obj[ObjNo].active = (Value); }
#define __hel_ObjSetX(ObjNo, X)             { ham_obj[ObjNo].objatr1 &= 0xFE00; ham_obj[ObjNo].objatr1 |= ((X) & 0x01FF); }
#define __hel_ObjSetY(ObjNo, Y)             { ham_obj[ObjNo].objatr0 &= 0xFF00; ham_obj[ObjNo].objatr0 |= ((Y) & 0x00FF); }
#define __hel_ObjSetXY(ObjNo, X, Y)         { __hel_ObjSetX(ObjNo, X); __hel_ObjSetY(ObjNo, Y); }
#define __hel_ObjSetHFlip(ObjNo, Value)     { ham_obj[ObjNo].objatr1 &= 0xEFFF; ham_obj[ObjNo].objatr1 |= ((Value) << 12); }
#define __hel_ObjSetVFlip(ObjNo, Value)     { ham_obj[ObjNo].objatr1 &= 0xDFFF; ham_obj[ObjNo].objatr1 |= ((Value) << 13); }
#define __hel_ObjSetPrio(ObjNo, Prio)       { ham_obj[ObjNo].objatr2 &= 0xF3FF; ham_obj[ObjNo].objatr2 |= ((Prio) << 10); }
#define __hel_ObjGetGfxSlot(ObjNo)          (ham_obj[ObjNo].objatr2 & 0x03ff)
#define __hel_ObjSetMode(ObjNo, Mode)       { ham_obj[ObjNo].objatr0 &= 0xf3ff; ham_obj[ObjNo].objatr0 |= ((Mode) << 10); }
#define __hel_ObjGetPrio(ObjNo)             ((ham_obj[ObjNo].objatr2 >> 10) & 3)

#ifdef HAM_DEBUGGER
    extern u8 hel_ObjExists(u8 ObjNo);
    extern u8 hel_ObjIsHFlip(u8 ObjNo);
    extern u8 hel_ObjIsVFlip(u8 ObjNo);
    extern u8 hel_ObjIsVisible(u8 ObjNo);
    extern u8 hel_ObjIsMosaic(u8 ObjNo);
    extern u8 hel_ObjIsRotScale(u8 ObjNo);
    extern u8 hel_ObjGetColorMode(u8 ObjNo);
    extern void hel_ObjBringToFront(u8 ObjNo);
    extern u8 hel_ObjGetFirst(void);
    extern void hel_ObjHideAll(void);
    extern void hel_ObjShowAll(void);
    extern u8 hel_ObjToggleVisible(u8 ObjNo);
    extern void hel_ObjSetVisible(u8 ObjNo, u8 Value);
    extern void hel_ObjSetX(u8 ObjNo, s16 X);
    extern void hel_ObjSetY(u8 ObjNo, s16 X);
    extern void hel_ObjSetXY(u8 ObjNo, s16 X, s16 Y);
    extern void hel_ObjSetHFlip(u8 ObjNo, u8 Value);
    extern void hel_ObjSetVFlip(u8 ObjNo, u8 Value);
    extern void hel_ObjSetPrio(u8 ObjNo, u32 Prio);
    extern u16 hel_ObjGetGfxSlot(u8 ObjNo);
    extern void hel_ObjSetMode(u8 ObjNo, u8 Mode);
    extern u8 hel_ObjGetPrio(u8 ObjNo);
#else
    #define hel_ObjExists(ObjNo)                __hel_ObjExists(ObjNo)
    #define hel_ObjIsHFlip(ObjNo)               __hel_ObjIsHFlip(ObjNo)
    #define hel_ObjIsVFlip(ObjNo)               __hel_ObjIsVFlip(ObjNo)
    #define hel_ObjIsVisible(ObjNo)             __hel_ObjIsVisible(ObjNo)
    #define hel_ObjIsMosaic(ObjNo)              __hel_ObjIsMosaic(ObjNo)
    #define hel_ObjIsRotScale(ObjNo)            __hel_ObjIsRotScale(ObjNo)
    #define hel_ObjGetColorMode(ObjNo)          __hel_ObjGetColorMode(ObjNo)
    #define hel_ObjGetMode(ObjNo)               __hel_ObjGetMode(ObjNo)
    #define hel_ObjBringToFront(ObjNo)          __hel_ObjBringToFront(ObjNo)
    #define hel_ObjGetFirst()                   __hel_ObjGetFirst()
    #define hel_ObjHideAll()                    __hel_ObjHideAll()
    #define hel_ObjShowAll()                    __hel_ObjShowAll()
    #define hel_ObjToggleVisible(ObjNo)         __hel_ObjToggleVisible(ObjNo)
    #define hel_ObjSetVisible(ObjNo, Value)     __hel_ObjSetVisible(ObjNo, Value)
    #define hel_ObjSetX(ObjNo, X)               __hel_ObjSetX(ObjNo, X)
    #define hel_ObjSetY(ObjNo, Y)               __hel_ObjSetY(ObjNo, Y)
    #define hel_ObjSetXY(ObjNo, X, Y)           __hel_ObjSetXY(ObjNo, X, Y)
    #define hel_ObjSetHFlip(ObjNo, Value)       __hel_ObjSetHFlip(ObjNo, Value)
    #define hel_ObjSetVFlip(ObjNo, Value)       __hel_ObjSetHFlip(ObjNo, Value)
    #define hel_ObjSetPrio(ObjNo, Prio)         __hel_ObjSetPrio(ObjNo, Prio)
    #define hel_ObjGetGfxSlot(ObjNo)            __hel_ObjGetGfxSlot(ObjNo)
    #define hel_ObjSetMode(ObjNo, Mode)         __hel_ObjSetMode(ObjNo, Mode)
    #define hel_ObjGetPrio(ObjNo)               __hel_ObjGetPrio(ObjNo)
#endif  // HAM_DEBUGGER


#define hel_ObjCalcFrameOffset(ObjNo, Index) (OBJ_SIZE_LUT[ham_obj[ObjNo].objatr0 >> 14][ham_obj[ObjNo].objatr1 >> 14] * Index) >> (!((ham_obj[ObjNo].objatr0 >> 13)&1))


extern u8 hel_ObjCreate(void* pSrc, u16 ObjShape, u16 ObjSize, u16 ObjMode, u16 ColMode, u16 PalNo, u16 Mosaic, u16 HFlip, u16 VFlip, u16 Prio, u16 DblSize, s16 X, s16 Y);
extern u8 hel_ObjCreate256(void* pSrc, u16 ObjShape, u16 ObjSize, u16 ObjMode, s16 X, s16 Y);
extern u8 hel_ObjCreate16(void* pSrc, u16 ObjShape, u16 ObjSize, u16 ObjMode, u16 PalNo, s16 X, s16 Y);
extern inline void hel_ObjUpdateInOAM(u8 ObjNo);
extern void hel_ObjSetVisibleAll(u8 Show);
extern u32 hel_ObjCountVisible(void);
extern inline void hel_ObjSetMosaicSize(u8 HSize, u8 VSize);
extern void hel_ObjSendToBack(u32 ObjNo);
extern void hel_ObjClearOAM(void);


//
// ---[ BG functions ]------------------------------------------------------------------------------
//

extern TPointU8 __hel_BgMosaicSize;

extern inline void hel_BgSetMosaic(u8 BgNo, u8 Value);
extern inline void hel_BgSetMosaicSize(u8 HSize, u8 VSize);


//
// ---[ DMA functions ]-----------------------------------------------------------------------------
//
extern void hel_DmaSet8(void *pDest, u8 SrcValue, u32 SizeInBytes);
extern void hel_DmaSet16(void *pDest, u16 SrcValue, u32 SizeIn16BitWords);
extern void hel_DmaCopy32(void *pDest, const void *pSrc, u32 SizeIn32BitWords);
extern void hel_DmaCopy16(void *pDest, const void *pSrc, u32 SizeIn16BitWords);
extern void hel_DmaSet32(void *pDest, u32 SrcValue, u32 SizeIn32BitWords);
extern void hel_DmaZeroMemory(void *pDest, u32 SizeInBytes);


//
// ---[ Sprite functions ]--------------------------------------------------------------------------
//

typedef struct _TSprite
{
   u8 ObjNo;
   s16 X,Y;
}TSprite, *PSprite;


//
// ---[ Splash functions ]--------------------------------------------------------------------------
//
extern void hel_Splash(void *pBitmap, void *pPalette, u8 SplashTypeIn, u8 SplashTypeOut, u8 FadeInSpeed, u8 FadeOutSpeed, u8 WaitFrames, u8 AfterWaitFrames);


//
// ---[ Fade functions ]----------------------------------------------------------------------------
//

typedef struct _TFadeInfo
{
   void *pPalette;
   u8 PalMode;
   u8 ColMode;
   u8 PalNo;
   u8 CurrentAmount;
}TFadeInfo, *PFadeInfo;


extern TFadeInfo hel_FadeInit(void *pPalette, u8 PalMode, u8 ColMode, u8 PalNo);
extern u8 hel_FadePalette(TFadeInfo *pFadeInfo, s8 AmountPerCall);
extern void hel_FadeReset(TFadeInfo *pFadeInfo);


//
// ---[ SRAM functions ]----------------------------------------------------------------------------
//

extern u8 hel_RAMEntryExists(char *pIdentifier);


//
// ---[ Interrupt functions ]-----------------------------------------------------------------------
//

typedef void (*PInterruptProc)(void);

extern u8 hel_IntrIsEnabled(u32 InterruptType);
extern void hel_IntrStopHandler(u32 InterruptType);
extern void hel_IntrStopAll(void);
extern u32 hel_IntrGetType(void *InterruptProc);

#ifdef HAM_DEBUGGER
    extern void __hel_IntrEmptyVBL(void);
    extern void __hel_IntrEmptyHBL(void);
    extern void __hel_IntrEmptyVCNT(void);
    extern void __hel_IntrEmptyTIM0(void);
    extern void __hel_IntrEmptyTIM1(void);
    extern void __hel_IntrEmptyTIM2(void);
    extern void __hel_IntrEmptyTIM3(void);
    extern void __hel_IntrEmptySIO(void);
    extern void __hel_IntrEmptyDMA0(void);
    extern void __hel_IntrEmptyDMA1(void);
    extern void __hel_IntrEmptyDMA2(void);
    extern void __hel_IntrEmptyDMA3(void);
    extern void __hel_IntrEmptyKEY(void);
    extern void __hel_IntrEmptyCART(void);
#endif // HAM_DEBUGGER


//
// ---[ Bitmap-Mode functions ]---------------------------------------------------------------------
//

extern void hel_BmpClear(u16 Color);
extern u16 *hel_BmpGetBackBuffer(void);


//
// ---[ Timer functions ]---------------------------------------------------------------------------
//

extern void hel_TimerSetFrequency(u8 TimerNo, u8 Value);
extern void hel_TimerStop(u8 TimerNo);
extern void hel_TimerStart(u8 TimerNo);
extern void hel_TimerSetEnable(u8 TimerNo, u8 Value);
extern u16 hel_TimerGetValue(u8 TimerNo);
extern void hel_TimerSetValue(u8 TimerNo, u16 Value);
extern void hel_TimerResetValue(u8 TimerNo);
extern u8 hel_TimerIsEnabled(u8 TimerNo);
extern void hel_TimerSetIRQEnable(u8 TimerNo, u8 Value);
extern u8 hel_TimerIsIRQEnabled(u8 TimerNo);
extern void hel_TimerSetCountUpEnable(u8 TimerNo, u8 Value);
extern u8 hel_TimerIsCountUpEnabled(u8 TimerNo);
extern void hel_TimerStopAll(void);


//
// ---[ HEL Assertion Implementation ]--------------------------------------------------------------
//

#ifdef HAM_DEBUGGER

  typedef struct
  {
      u8 Width;
      u8 PitchY;
      char *pCharData;
  }TAssertCharInfo;

  extern const TAssertCharInfo *__hel_AssertCharInfos[];
  extern void hel_AssertImplementation(u8 Assertion, char *pExpression, char *pFilename,
                                       char *pFunction, u32 LineNo, char *pMessage, ...);

  #define HEL_ASSERT(expression, message ...)      \
      if(!(expression))                            \
          hel_AssertImplementation((expression), #expression, __FILE__, __PRETTY_FUNCTION__, __LINE__, message);
        
#else

  #define HEL_ASSERT(assertion, message, args ...)

#endif // HAM_DEBUGGER


//
// ---[ System Functions ]--------------------------------------------------------------------------
//

#define _hel_SysSetPrefetch(Value)    { R_WAITCNT |= (Value << 14); }

#ifdef HAM_DEBUGGER
  extern void hel_SysSetPrefetch(u32 Value);
#else
  #define hel_SysSetPrefetch(Value)   _hel_SysSetPrefetch(Value)
#endif


#ifdef __cplusplus
   }
#endif

#endif // HEL_H


/* END OF FILE */

